@@ -1,10 +1,11 @@
 import asyncio
 import enum
+import functools
 import threading
 import typing
 import weakref
 
-from .utils import GlobalEvent, AutoincrementDWORD, no_async
+from .utils import GlobalEvent, AutoincrementDWORD, AsyncThreadingLock, no_async
 from .base import HasExceptions
 from .data import DataField, DataRequest
 from .events import SystemEvent, SystemEventType
@@ -34,7 +35,7 @@ class Sim:
     _sim: typing.Optional[_native.Sim] = None
     _loop_thr: typing.Optional[threading.Thread] = None
     _stopping: bool = False
-    _lock: threading.Lock  # Instance-wide consistency lock
+    _lock: AsyncThreadingLock  # Instance-wide consistency lock
     _open_evt: GlobalEvent
     _stop_evt: GlobalEvent
     _loop_done_evt = GlobalEvent
@@ -51,7 +52,7 @@ class Sim:
         self._loop_done_evt = GlobalEvent(packet_ids=[-1])  # FIXME: make packet_ids optional?
         self._stop_evt = GlobalEvent(packet_ids=[-1])
         self._stop_evt.set()
-        self._lock = threading.Lock()
+        self._lock = AsyncThreadingLock()
 
         self._event_id_inc = AutoincrementDWORD()
         self._data_request_id_inc = AutoincrementDWORD()
@@ -120,19 +121,22 @@ class Sim:
     async def wait_stop(self, timeout: typing.Optional[float] = None) -> None:
         await self._stop_evt.wait(timeout)
 
-    def subscribe_to_system_event(self, event_type: events.SystemEventType) -> SystemEvent:
+    async def subscribe_to_system_event(self, event_type: events.SystemEventType) -> SystemEvent:
         # TODO: Add explicit unsubscribe
         # No locking here. Hoping that the counter will not wrap around and reach our value again
         # before the code below completes.
         event_id = self._event_id_inc.next(self._events)
 
         def clean_event(_):
-            self._sim.unsubscribe_from_system_event(event_id)
-            self._events.pop(event_id)
-            self._exception_targets.pop(packet_id)
+            try:
+                self.sim.unsubscribe_from_system_event(event_id)
+            except NotStarted:
+                pass
+            self._events.pop(event_id, None)
+            self._exception_targets.pop(packet_id, None)
             print(f"Cleaned event #{event_id}")
 
-        with self._lock:
+        async with self._lock:
             self.sim.subscribe_to_system_event(event_id, event_type)
             packet_id = self.sim.get_last_sent_packet_id()
             event = SystemEvent(packet_ids=[packet_id], event_id=event_id)
@@ -143,10 +147,10 @@ class Sim:
         return event
 
     async def request_data_once(self, fields: typing.List[DataField], object_id: int = 0, timeout: typing.Optional[float] = None):
-        request = self.request_data_on_sim_object(fields, PERIOD.ONCE, object_id)
+        request = await self.request_data_on_sim_object(fields, PERIOD.ONCE, object_id)
         return await request.wait(timeout)
 
-    def request_data_on_sim_object(
+    async def request_data_on_sim_object(
             self, fields: typing.List[DataField], period: PERIOD, object_id: int = 0, when: DataWhen = DataWhen.Always,
             origin: int = 0, interval: int = 0, limit: int = 0):
         # TODO: Add explicit unsubscribe
@@ -157,12 +161,12 @@ class Sim:
         def clean_data_request(_):
             # TODO: unsubscribe from data!
             self._data_definition_ids.discard(definition_id)
-            self._data_requests.pop(request_id)
+            self._data_requests.pop(request_id, None)
             for packet_id in packet_ids:
-                self._exception_targets.pop(packet_id)
+                self._exception_targets.pop(packet_id, None)
             print(f"Cleaned data request #{request_id}")
 
-        with self._lock:
+        async with self._lock:
             for field in fields:
                 self.sim.add_to_data_definition(definition_id, field.name, field.units, field.type.data_type)
                 packet_ids.append(self.sim.get_last_sent_packet_id())
@@ -222,17 +226,23 @@ class Sim:
         self._stopping = True
         self._loop_thr = None
 
-        for event_ref in self._events.values():
+        while True:
+            try:
+                _, event_ref = self._events.popitem()
+            except KeyError:
+                break
             event = event_ref()
             if event is not None:
                 event.close()
-        self._events = {}
 
-        for data_request_ref in self._data_requests.values():
+        while True:
+            try:
+                _, data_request_ref = self._data_requests.popitem()
+            except KeyError:
+                break
             data_request = data_request_ref()
             if data_request is not None:
                 data_request.close()
-        self._data_requests = {}
 
         self._data_definition_ids = set()
         self._open_evt.clear()
@@ -280,21 +290,36 @@ class Sim:
         return asyncio.run(self.__aexit__(exc_type, exc_val, exc_tb))
 
     @no_async
-    def sync_wait_open(self, timeout: typing.Optional[float] = OPEN_TIMEOUT_SEC) -> None:
-        return asyncio.run(self.wait_open(timeout))
+    @functools.wraps(wait_open)
+    def sync_wait_open(self, *args, **kwargs):
+        return asyncio.run(self.wait_open(*args, **kwargs))
 
     @no_async
-    def sync_wait_stop(self, timeout: typing.Optional[float] = None) -> None:
-        return asyncio.run(self.wait_stop(timeout))
+    @functools.wraps(wait_stop)
+    def sync_wait_stop(self, *args, **kwargs):
+        return asyncio.run(self.wait_stop(*args, **kwargs))
 
     @no_async
-    def sync_request_data_once(self, fields: typing.List[DataField], object_id: int = 0, timeout: typing.Optional[float] = None):
-        return asyncio.run(self.request_data_once(fields, object_id, timeout))
+    @functools.wraps(request_data_once)
+    def sync_request_data_once(self, *args, **kwargs):
+        return asyncio.run(self.request_data_once(*args, **kwargs))
 
     @no_async
-    def sync_start(self, open_timeout: typing.Optional[float] = OPEN_TIMEOUT_SEC):
-        return asyncio.run(self.start(open_timeout))
+    @functools.wraps(request_data_on_sim_object)
+    def sync_request_data_on_sim_object(self, *args, **kwargs):
+        return asyncio.run(self.request_data_on_sim_object(*args, **kwargs))
 
     @no_async
-    def sync_stop(self):
-        return asyncio.run(self.stop())
+    @functools.wraps(subscribe_to_system_event)
+    def sync_subscribe_to_system_event(self, *args, **kwargs):
+        return asyncio.run(self.subscribe_to_system_event(*args, **kwargs))
+
+    @no_async
+    @functools.wraps(start)
+    def sync_start(self, *args, **kwargs):
+        return asyncio.run(self.start(*args, **kwargs))
+
+    @no_async
+    @functools.wraps(stop)
+    def sync_stop(self, *args, **kwargs):
+        return asyncio.run(self.stop(*args, **kwargs))
